package xyz.wbsite.zentao.sdk;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONUtil;
import xyz.wbsite.zentao.sdk.execution.req.ZtExecutionCreateRequest;
import xyz.wbsite.zentao.sdk.execution.req.ZtExecutionStartRequest;
import xyz.wbsite.zentao.sdk.execution.rsp.ZtExecutionCreateResponse;
import xyz.wbsite.zentao.sdk.execution.rsp.ZtExecutionStartResponse;
import xyz.wbsite.zentao.sdk.project.req.ZtProjectActivateRequest;
import xyz.wbsite.zentao.sdk.project.req.ZtProjectCloseRequest;
import xyz.wbsite.zentao.sdk.project.req.ZtProjectCreateRequest;
import xyz.wbsite.zentao.sdk.project.req.ZtProjectDeleteRequest;
import xyz.wbsite.zentao.sdk.project.req.ZtProjectEditRequest;
import xyz.wbsite.zentao.sdk.project.req.ZtProjectStartRequest;
import xyz.wbsite.zentao.sdk.project.rsp.ZtProjectActivateResponse;
import xyz.wbsite.zentao.sdk.project.rsp.ZtProjectCloseResponse;
import xyz.wbsite.zentao.sdk.project.rsp.ZtProjectCreateResponse;
import xyz.wbsite.zentao.sdk.project.rsp.ZtProjectDeleteResponse;
import xyz.wbsite.zentao.sdk.project.rsp.ZtProjectEditResponse;
import xyz.wbsite.zentao.sdk.project.rsp.ZtProjectStartResponse;
import xyz.wbsite.zentao.sdk.task.req.ZtTaskCloseRequest;
import xyz.wbsite.zentao.sdk.task.req.ZtTaskCreateRequest;
import xyz.wbsite.zentao.sdk.task.req.ZtTaskDeleteRequest;
import xyz.wbsite.zentao.sdk.task.req.ZtTaskFinishRequest;
import xyz.wbsite.zentao.sdk.task.req.ZtTaskStartRequest;
import xyz.wbsite.zentao.sdk.task.rsp.ZtTaskCloseResponse;
import xyz.wbsite.zentao.sdk.task.rsp.ZtTaskCreateResponse;
import xyz.wbsite.zentao.sdk.task.rsp.ZtTaskDeleteResponse;
import xyz.wbsite.zentao.sdk.task.rsp.ZtTaskFinishResponse;
import xyz.wbsite.zentao.sdk.task.rsp.ZtTaskGetResponse;
import xyz.wbsite.zentao.sdk.task.rsp.ZtTaskStartResponse;
import org.junit.Test;

import java.util.Collections;

/**
 * 禅道客户端测试
 * <p>
 * 基于开源版本18.1开发
 *
 * @since 1.8
 */
public class ClientTest {

    @Test
    public void test() {
        // 初始化客户端
        ZentaoClient client = new ZentaoClient("http://127.0.0.1:80", "******", "********************************", "pms");
        ZentaoInstance instance = client.loginAsPms();

        // 创建项目
        ZtProjectCreateRequest ztProjectCreateRequest = new ZtProjectCreateRequest();
        ztProjectCreateRequest.setParent("0");
        ztProjectCreateRequest.setName("测试项目");
        ztProjectCreateRequest.setCode("测试项目");
        ztProjectCreateRequest.setMultiple("1");
        ztProjectCreateRequest.setHasProduct("0");
        ztProjectCreateRequest.setPm("");
        ztProjectCreateRequest.setBudget("1000");
        ztProjectCreateRequest.setBudgetUnit("CNY");
        ztProjectCreateRequest.setBegin(DateUtil.date().toString("yyyy-MM-dd"));
        ztProjectCreateRequest.setEnd(DateUtil.date().offset(DateField.DAY_OF_MONTH, 27).toString("yyyy-MM-dd"));
        ztProjectCreateRequest.setDays(21);
        ztProjectCreateRequest.setProducts("0");
        ztProjectCreateRequest.setProductName("");
        ztProjectCreateRequest.setDesc("");
        ztProjectCreateRequest.setAcl("open");
        ztProjectCreateRequest.setAuth("extend");
        ztProjectCreateRequest.setModel("scrum");
        ZtProjectCreateResponse createProjectResponse = instance.getZtProjectService().create(ztProjectCreateRequest);
        System.out.println(JSONUtil.toJsonPrettyStr(createProjectResponse));

        // 编辑项目
        ZtProjectEditRequest ztProjectEditRequest = new ZtProjectEditRequest();
        ztProjectEditRequest.setId(createProjectResponse.getId());
        ztProjectEditRequest.setParent("0");
        ztProjectEditRequest.setName("测试项目");
        ztProjectEditRequest.setCode("测试项目");
        ztProjectEditRequest.setPm("");
        ztProjectEditRequest.setBudget("1000");
        ztProjectEditRequest.setBudgetUnit("CNY");
        ztProjectEditRequest.setBegin(DateUtil.date().toString("yyyy-MM-dd"));
        ztProjectEditRequest.setEnd(DateUtil.date().offset(DateField.DAY_OF_MONTH, 27).toString("yyyy-MM-dd"));
        ztProjectEditRequest.setDays(21);
        ztProjectEditRequest.setDesc("");
        ztProjectEditRequest.setAcl("open");
        ztProjectEditRequest.setAuth("extend");
        ztProjectEditRequest.setModel("scrum");
        ZtProjectEditResponse ztProjectEditResponse = instance.getZtProjectService().edit(ztProjectEditRequest);
        System.out.println(JSONUtil.toJsonPrettyStr(ztProjectEditResponse));

        // 开始项目
        ZtProjectStartRequest ztProjectStartRequest = new ZtProjectStartRequest();
        ztProjectStartRequest.setProjectId(createProjectResponse.getId());
        ztProjectStartRequest.setComment("");
        ztProjectStartRequest.setRealBegan(DateUtil.today());
        ZtProjectStartResponse startProjectResponse = instance.getZtProjectService().start(ztProjectStartRequest);
        System.out.println(JSONUtil.toJsonPrettyStr(startProjectResponse));

        // 创建执行（模块1）
        {
            ZtExecutionCreateRequest ztExecutionCreateRequest = new ZtExecutionCreateRequest();
            ztExecutionCreateRequest.setProject(createProjectResponse.getId());
            ztExecutionCreateRequest.setName("测试项目-模块1");
            ztExecutionCreateRequest.setCode("测试项目-模块1");
            ztExecutionCreateRequest.setBegin(DateUtil.date().toString("yyyy-MM-dd"));
            ztExecutionCreateRequest.setEnd(DateUtil.date().offset(DateField.DAY_OF_MONTH, 6).toString("yyyy-MM-dd"));
            ztExecutionCreateRequest.setDelta(7);
            ztExecutionCreateRequest.setDays(5);
            ztExecutionCreateRequest.setLifetime("short");
            ztExecutionCreateRequest.setTeam("");
            ztExecutionCreateRequest.setTeams(createProjectResponse.getId());
            ztExecutionCreateRequest.setPo("");
            ztExecutionCreateRequest.setQd("");
            ztExecutionCreateRequest.setPm("");
            ztExecutionCreateRequest.setRd("");
            ztExecutionCreateRequest.setTeamMembers(CollUtil.newArrayList(""));
            ztExecutionCreateRequest.setDesc("");
            ztExecutionCreateRequest.setAcl("open");
            ztExecutionCreateRequest.setAuth("extend");
            ZtExecutionCreateResponse ztExecutionCreateResponse = instance.getZtExecutionService().create(ztExecutionCreateRequest);
            System.out.println(JSONUtil.toJsonPrettyStr(ztExecutionCreateResponse));

            // 开始执行（模块1）
            ZtExecutionStartRequest ztExecutionStartRequest = new ZtExecutionStartRequest();
            ztExecutionStartRequest.setExecutionId(ztExecutionCreateResponse.getId());
            ztExecutionStartRequest.setComment("");
            ztExecutionStartRequest.setRealBegan(DateUtil.today());
            ZtExecutionStartResponse ztExecutionStartResponse = instance.getZtExecutionService().start(ztExecutionStartRequest);
            System.out.println(JSONUtil.toJsonPrettyStr(ztExecutionStartResponse));

            {
                // 创建测试任务1
                ZtTaskCreateRequest ztTaskCreateRequest = new ZtTaskCreateRequest();
                ztTaskCreateRequest.setExecution(ztExecutionCreateResponse.getId());
                ztTaskCreateRequest.setType("devel");
                ztTaskCreateRequest.setModule("0");
                ztTaskCreateRequest.setAssignedTo(Collections.emptyList());
                ztTaskCreateRequest.setTeamMember("");
                ztTaskCreateRequest.setMultiple("");
                ztTaskCreateRequest.setMode("");
                ztTaskCreateRequest.setStory("");
                ztTaskCreateRequest.setStoryEstimate("");
                ztTaskCreateRequest.setStoryDesc("");
                ztTaskCreateRequest.setStoryPri("");
                ztTaskCreateRequest.setColor("#ff0000");
                ztTaskCreateRequest.setName("测试任务1");
                ztTaskCreateRequest.setPri("1");
                ztTaskCreateRequest.setEstimate("1");
                ztTaskCreateRequest.setDesc("任务描述");
                ztTaskCreateRequest.setEstStarted(DateUtil.date().toString("yyyy-MM-dd"));
                ztTaskCreateRequest.setDeadline(DateUtil.date().toString("yyyy-MM-dd"));
                ztTaskCreateRequest.setMailtos(Collections.emptyList());
                ZtTaskCreateResponse ztTaskCreateResponse = instance.getZtTaskService().create(ztTaskCreateRequest);
                System.out.println(JSONUtil.toJsonPrettyStr(ztTaskCreateResponse));

                ZtTaskGetResponse ztTaskGetResponse = instance.getZtTaskService().getByName(ztExecutionCreateResponse.getId(), "测试任务1");
                // 开始任务
                ZtTaskStartRequest ztTaskStartRequest = new ZtTaskStartRequest();
                ztTaskStartRequest.setTaskId(ztTaskGetResponse.getResult().getId());
                ztTaskStartRequest.setAssignedTo("");
                ztTaskStartRequest.setRealStarted(DateUtil.date().toString("yyyy-MM-dd HH:mm:ss"));
                ztTaskStartRequest.setConsumed(ztTaskGetResponse.getResult().getConsumed());
                ztTaskStartRequest.setLeft(ztTaskGetResponse.getResult().getLeft());
                ztTaskStartRequest.setComment("备注");
                ZtTaskStartResponse ztTaskStartResponse = instance.getZtTaskService().start(ztTaskStartRequest);
                System.out.println(JSONUtil.toJsonPrettyStr(ztTaskStartResponse));

                // 完成任务
                ZtTaskFinishRequest ztTaskFinishRequest = new ZtTaskFinishRequest();
                ztTaskFinishRequest.setTaskId(ztTaskGetResponse.getResult().getId());
                ztTaskFinishRequest.setAssignedTo("");
                ztTaskFinishRequest.setRealStarted(DateUtil.date().toString("yyyy-MM-dd HH:mm:ss"));
                ztTaskFinishRequest.setConsumed(ztTaskGetResponse.getResult().getConsumed());
                ztTaskFinishRequest.setComment("备注");
                ZtTaskFinishResponse ztTaskFinishResponse = instance.getZtTaskService().finish(ztTaskFinishRequest);
                System.out.println(JSONUtil.toJsonPrettyStr(ztTaskFinishResponse));

                // 关闭任务
                ZtTaskCloseRequest ztTaskCloseRequest = new ZtTaskCloseRequest();
                ztTaskCloseRequest.setTaskId(ztTaskCreateResponse.getId());
                ztTaskCloseRequest.setComment("备注");
                ZtTaskCloseResponse ztTaskCloseResponse = instance.getZtTaskService().close(ztTaskCloseRequest);
                System.out.println(JSONUtil.toJsonPrettyStr(ztTaskCloseResponse));

                // 删除任务
                ZtTaskDeleteRequest ztTaskDeleteRequest = new ZtTaskDeleteRequest();
                ztTaskDeleteRequest.setTaskId(ztTaskCreateResponse.getId());
                ZtTaskDeleteResponse ztTaskDeleteResponse = instance.getZtTaskService().delete(ztTaskDeleteRequest);
                System.out.println(JSONUtil.toJsonPrettyStr(ztTaskDeleteResponse));
            }
        }

        // 关闭项目
        ZtProjectCloseRequest ztProjectCloseRequest = new ZtProjectCloseRequest();
        ztProjectCloseRequest.setProjectId(createProjectResponse.getId());
        ztProjectCloseRequest.setComment("");
        ztProjectCloseRequest.setRealEnd(DateUtil.today());
        ZtProjectCloseResponse closeProjectResponse = instance.getZtProjectService().close(ztProjectCloseRequest);
        System.out.println(JSONUtil.toJsonPrettyStr(closeProjectResponse));

        // 激活项目
        ZtProjectActivateRequest ztProjectActivateRequest = new ZtProjectActivateRequest();
        ztProjectActivateRequest.setProjectId(createProjectResponse.getId());
        ztProjectActivateRequest.setComment("");
        ztProjectActivateRequest.setBegin(DateUtil.today());
        ZtProjectActivateResponse activateProjectResponse = instance.getZtProjectService().activate(ztProjectActivateRequest);
        System.out.println(JSONUtil.toJsonPrettyStr(activateProjectResponse));

        // 删除项目
        ZtProjectDeleteRequest ztProjectDeleteRequest = new ZtProjectDeleteRequest();
        ztProjectDeleteRequest.setProjectId(createProjectResponse.getId());
        ZtProjectDeleteResponse ztProjectDeleteResponse = instance.getZtProjectService().delete(ztProjectDeleteRequest);
        System.out.println(JSONUtil.toJsonPrettyStr(ztProjectDeleteResponse));
    }
}
